Fixing merge conflict in Quickstart section.
[privoxy.git] / gateway.c
index 2e36abb..7584b94 100644 (file)
--- a/gateway.c
+++ b/gateway.c
@@ -1,4 +1,4 @@
-const char gateway_rcs[] = "$Id: gateway.c,v 1.8 2001/09/13 20:10:12 jongfoster Exp $";
+const char gateway_rcs[] = "$Id: gateway.c,v 1.15 2002/03/26 22:29:54 swa Exp $";
 /*********************************************************************
  *
  * File        :  $Source: /cvsroot/ijbswa/current/gateway.c,v $
@@ -8,7 +8,7 @@ const char gateway_rcs[] = "$Id: gateway.c,v 1.8 2001/09/13 20:10:12 jongfoster
  *                proxy).
  *
  * Copyright   :  Written by and Copyright (C) 2001 the SourceForge
- *                IJBSWA team.  http://ijbswa.sourceforge.net
+ *                Privoxy team. http://www.privoxy.org/
  *
  *                Based on the Internet Junkbuster originally written
  *                by and Copyright (C) 1997 Anonymous Coders and
@@ -34,6 +34,52 @@ const char gateway_rcs[] = "$Id: gateway.c,v 1.8 2001/09/13 20:10:12 jongfoster
  *
  * Revisions   :
  *    $Log: gateway.c,v $
+ *    Revision 1.15  2002/03/26 22:29:54  swa
+ *    we have a new homepage!
+ *
+ *    Revision 1.14  2002/03/24 13:25:43  swa
+ *    name change related issues
+ *
+ *    Revision 1.13  2002/03/13 00:29:59  jongfoster
+ *    Killing warnings
+ *
+ *    Revision 1.12  2002/03/09 20:03:52  jongfoster
+ *    - Making various functions return int rather than size_t.
+ *      (Undoing a recent change).  Since size_t is unsigned on
+ *      Windows, functions like read_socket that return -1 on
+ *      error cannot return a size_t.
+ *
+ *      THIS WAS A MAJOR BUG - it caused frequent, unpredictable
+ *      crashes, and also frequently caused JB to jump to 100%
+ *      CPU and stay there.  (Because it thought it had just
+ *      read ((unsigned)-1) == 4Gb of data...)
+ *
+ *    - The signature of write_socket has changed, it now simply
+ *      returns success=0/failure=nonzero.
+ *
+ *    - Trying to get rid of a few warnings --with-debug on
+ *      Windows, I've introduced a new type "jb_socket".  This is
+ *      used for the socket file descriptors.  On Windows, this
+ *      is SOCKET (a typedef for unsigned).  Everywhere else, it's
+ *      an int.  The error value can't be -1 any more, so it's
+ *      now JB_INVALID_SOCKET (which is -1 on UNIX, and in
+ *      Windows it maps to the #define INVALID_SOCKET.)
+ *
+ *    - The signature of bind_port has changed.
+ *
+ *    Revision 1.11  2002/03/08 17:46:04  jongfoster
+ *    Fixing int/size_t warnings
+ *
+ *    Revision 1.10  2002/03/07 03:50:19  oes
+ *     - Improved handling of failed DNS lookups
+ *     - Fixed compiler warnings
+ *
+ *    Revision 1.9  2001/10/25 03:40:48  david__schmidt
+ *    Change in porting tactics: OS/2's EMX porting layer doesn't allow multiple
+ *    threads to call select() simultaneously.  So, it's time to do a real, live,
+ *    native OS/2 port.  See defines for __EMX__ (the porting layer) vs. __OS2__
+ *    (native). Both versions will work, but using __OS2__ offers multi-threading.
+ *
  *    Revision 1.8  2001/09/13 20:10:12  jongfoster
  *    Fixing missing #include under Windows
  *
@@ -103,10 +149,10 @@ const char gateway_rcs[] = "$Id: gateway.c,v 1.8 2001/09/13 20:10:12 jongfoster
 
 const char gateway_h_rcs[] = GATEWAY_H_VERSION;
 
-static int socks4_connect(const struct forward_spec * fwd,
-                          const char * target_host,
-                          int target_port,
-                          struct client_state *csp);
+static jb_socket socks4_connect(const struct forward_spec * fwd,
+                                const char * target_host,
+                                int target_port,
+                                struct client_state *csp);
 
 
 #define SOCKS_REQUEST_GRANTED          90
@@ -143,16 +189,16 @@ static const char socks_userid[] = "anonymous";
  *                a HTTP proxy and/or a SOCKS proxy.
  *
  * Parameters  :
- *          1  :  gw = pointer to a gateway structure (such as gw_default)
+ *          1  :  fwd = the proxies to use when connecting.
  *          2  :  http = the http request and apropos headers
  *          3  :  csp = Current client state (buffers, headers, etc...)
  *
* Returns     :  -1 => failure, else it is the socket file descriptor.
  *
  *********************************************************************/
-int forwarded_connect(const struct forward_spec * fwd,
-                      struct http_request *http,
-                      struct client_state *csp)
+jb_socket forwarded_connect(const struct forward_spec * fwd,
+                            struct http_request *http,
+                            struct client_state *csp)
 {
    const char * dest_host;
    int dest_port;
@@ -185,7 +231,7 @@ int forwarded_connect(const struct forward_spec * fwd,
          /* Should never get here */
          log_error(LOG_LEVEL_FATAL, "SOCKS4 impossible internal error - bad SOCKS type.");
          errno = EINVAL;
-         return(-1);
+         return(JB_INVALID_SOCKET);
    }
 }
 
@@ -201,26 +247,27 @@ int forwarded_connect(const struct forward_spec * fwd,
  *                normal (non-SOCKS) socket.
  *
  * Parameters  :
- *          1  :  gw = pointer to a gateway structure (such as gw_default)
- *          2  :  http = the http request and apropos headers
- *          3  :  csp = Current client state (buffers, headers, etc...)
+ *          1  :  fwd = Specifies the SOCKS proxy to use.
+ *          2  :  target_host = The final server to connect to.
+ *          3  :  target_port = The final port to connect to.
+ *          4  :  csp = Current client state (buffers, headers, etc...)
  *
* Returns     :  -1 => failure, else a socket file descriptor.
  *
  *********************************************************************/
-static int socks4_connect(const struct forward_spec * fwd,
-                          const char * target_host,
-                          int target_port,
-                          struct client_state *csp)
+static jb_socket socks4_connect(const struct forward_spec * fwd,
+                                const char * target_host,
+                                int target_port,
+                                struct client_state *csp)
 {
    int web_server_addr;
-   unsigned char cbuf[BUFFER_SIZE];
-   unsigned char sbuf[BUFFER_SIZE];
+   char cbuf[BUFFER_SIZE];
+   char sbuf[BUFFER_SIZE];
    struct socks_op    *c = (struct socks_op    *)cbuf;
    struct socks_reply *s = (struct socks_reply *)sbuf;
-   int n;
-   int csiz;
-   int sfd;
+   size_t n;
+   size_t csiz;
+   jb_socket sfd;
    int err = 0;
    char *errstr;
 
@@ -239,7 +286,7 @@ static int socks4_connect(const struct forward_spec * fwd,
    if (err)
    {
       errno = EINVAL;
-      return(-1);
+      return(JB_INVALID_SOCKET);
    }
 
    /* build a socks request for connection to the web server */
@@ -252,6 +299,11 @@ static int socks4_connect(const struct forward_spec * fwd,
    {
       case SOCKS_4:
          web_server_addr = htonl(resolve_hostname_to_ip(target_host));
+         if (web_server_addr == INADDR_NONE)
+         {
+            log_error(LOG_LEVEL_CONNECT, "socks4_connect: could not resolve target host %s", target_host);
+            return(JB_INVALID_SOCKET);
+         }
          break;
       case SOCKS_4A:
          web_server_addr = 0x00000001;
@@ -259,16 +311,16 @@ static int socks4_connect(const struct forward_spec * fwd,
          if (n > sizeof(cbuf))
          {
             errno = EINVAL;
-            return(-1);
+            return(JB_INVALID_SOCKET);
          }
-         strcpy(((char *)cbuf) + csiz, target_host);
+         strcpy(cbuf + csiz, target_host);
          csiz = n;
          break;
       default:
          /* Should never get here */
          log_error(LOG_LEVEL_FATAL, "SOCKS4 impossible internal error - bad SOCKS type.");
          errno = EINVAL;
-         return(-1);
+         return(JB_INVALID_SOCKET);
    }
 
    c->vn          = 4;
@@ -283,23 +335,23 @@ static int socks4_connect(const struct forward_spec * fwd,
    /* pass the request to the socks server */
    sfd = connect_to(fwd->gateway_host, fwd->gateway_port, csp);
 
-   if (sfd < 0)
+   if (sfd == JB_INVALID_SOCKET)
    {
-      return(-1);
+      return(JB_INVALID_SOCKET);
    }
 
-   if ((n = write_socket(sfd, (char *)c, csiz)) != csiz)
+   if (write_socket(sfd, (char *)c, csiz))
    {
       log_error(LOG_LEVEL_CONNECT, "SOCKS4 negotiation write failed...");
       close_socket(sfd);
-      return(-1);
+      return(JB_INVALID_SOCKET);
    }
 
-   if ((n = read_socket(sfd, sbuf, sizeof(sbuf))) != sizeof(*s))
+   if (read_socket(sfd, sbuf, sizeof(sbuf)) != sizeof(*s))
    {
       log_error(LOG_LEVEL_CONNECT, "SOCKS4 negotiation read failed...");
       close_socket(sfd);
-      return(-1);
+      return(JB_INVALID_SOCKET);
    }
 
    switch (s->cd)
@@ -323,7 +375,7 @@ static int socks4_connect(const struct forward_spec * fwd,
          errno = EACCES;
          break;
       default:
-         errstr = (char *) cbuf;
+         errstr = cbuf;
          errno = ENOENT;
          sprintf(errstr,
                  "SOCKS request rejected for reason code %d\n", s->cd);
@@ -332,7 +384,7 @@ static int socks4_connect(const struct forward_spec * fwd,
    log_error(LOG_LEVEL_CONNECT, "socks4_connect: %s ...", errstr);
 
    close_socket(sfd);
-   return(-1);
+   return(JB_INVALID_SOCKET);
 
 }
@@ -143,16 +189,16 @@ static const char socks_userid[] = "anonymous";
  *          3  :  csp = Current client state (buffers, headers, etc...)
  *
- * Returns     :  -1 => failure, else it is the socket file descriptor.
+ * Returns     :  JB_INVALID_SOCKET => failure, else it is the socket file descriptor.
  *
  *********************************************************************/
@@ -201,26 +247,27 @@ int forwarded_connect(const struct forward_spec * fwd,
  *
- * Returns     :  -1 => failure, else a socket file descriptor.
+ * Returns     :  JB_INVALID_SOCKET => failure, else a socket file descriptor.
  *
  *********************************************************************/